/*
 *    GS2MGRetrieve.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;


// Greenstone classes
import org.greenstone.mg.*;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Element;

// General Java classes
import java.io.File;


/**
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 7823 $
 */

public class GS2MGRetrieve
    extends GS2Retrieve {

    // Parameters used
    private static final String INDEX_PARAM = "index";

    // Elements used in the config file that are specific to this class
    private static final String DEFAULT_INDEX_ELEM = "defaultIndex";

    private MGWrapper mg_src = null;

    private String default_index = null;


    /** constructor */
    public GS2MGRetrieve()
    {
	this.mg_src = new MGWrapper();
    }


    /** configure this service */
    public boolean configure(Element info, Element extra_info)
    {
	// Do specific configuration
	System.out.println("Configuring GS2MGRetrieve...");
	// System.out.println("info:\n" + converter_.getString(info));
	// System.out.println("extra_info:\n" + converter_.getString(extra_info));

	// Get the default index out of <defaultIndex> (buildConfig.xml)
	Element def = (Element) GSXML.getChildByTagName(info, DEFAULT_INDEX_ELEM);
	if (def != null) {
	    this.default_index = def.getAttribute(GSXML.NAME_ATT);
	}
	if (this.default_index == null || this.default_index.equals("")) {
	    System.err.println("Error: default index not specified!");
	    return false;
	}
	// System.out.println("Default index: " + this.default_index);

	// Do generic configuration
	return super.configure(info, extra_info);
    }


    /** Retrieve the content of a document */
    protected Element processDocumentContentRetrieve(Element request)
    {
	// Create a new (empty) result message
	Element result = this.doc.createElement(GSXML.RESPONSE_ELEM);
	result.setAttribute(GSXML.FROM_ATT, DOCUMENT_CONTENT_RETRIEVE_SERVICE);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);

	// Get the parameters of the request - no parameters at this stage
  	//Element param_list = (Element) GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);

	// Get the request content 
	
	Element query_doc_list = (Element) GSXML.getChildByTagName(request, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	if (query_doc_list == null) {
	    System.err.println("Error: DocumentContentRetrieve request specified no doc nodes.\n");
	    return result;
	}
	
	Element doc_list = this.doc.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	result.appendChild(doc_list);
	
	// The location of the MG index and text files
	String basedir = GSFile.collectionBaseDir(this.site_home, this.cluster_name) + File.separatorChar;  // Needed by MG
	String textdir = GSFile.collectionTextPath(this.cluster_name);
	// index is only needed to start up MG, not used so just use the default index
	String indexpath = GSFile.collectionIndexPath(this.cluster_name, this.default_index);
	this.mg_src.setIndex(indexpath);
	
	// Get the documents
	String[] doc_ids = GSXML.getAttributeValuesFromList(query_doc_list, 
							     GSXML.NODE_ID_ATT);
	for (int i = 0; i < doc_ids.length; i++) {
	    String doc_id = doc_ids[i];
	    if (OID.needsTranslating(doc_id)) {
		doc_id = this.gdbm_src.translateOID(doc_id);
	    }
	    long doc_num = this.gdbm_src.oid2Docnum(doc_id);

	    String doc_content = this.mg_src.getDocument(basedir, textdir, doc_num);
	    // Create the  text node
	    Element doc = this.doc.createElement(GSXML.DOC_NODE_ELEM);
	    doc.setAttribute(GSXML.NODE_ID_ATT, doc_id);
	    doc_list.appendChild(doc);
	    
	    if (doc_content!=null) {
		System.err.println("got the content");
		// remove any ctrl-c or ctrl-b (I hope these are the right codes)
		//System.err.println("doc content "+doc_content);
		doc_content = doc_content.replaceAll("\u0002|\u0003", "");
		// replace _httpimg_ with the correct address
		doc_content = resolveImages(doc_content, doc_id);
		GSXML.addDocText(this.doc, doc, doc_content);
	    } else {
		System.err.println("the doc content was null, not getting that section\n");
		GSXML.addDocText(this.doc, doc, "couldn't retrieve content for this section\n");
	    }
	}

	return result;
    }
}
